module top(
	input clk_50mhz,				// pin:   L1
	
	input uart_rx,					// pin: F14
	output uart_tx,				// pin: G12
	
	output [6:0] hex_display1,	// pins: E2, F1, F2, H1, H2, J1, J2
	output [6:0] hex_display2, // pins: D1, D2, G3, H4, H5, H6, E1
	output [6:0] hex_display3, // pins: D3, E4, E3, C1, C2, G6, G5
	output [6:0] hex_display4	// pins: D4, F3, L8, J4, D6, D5, F4
);

// Generate reset signal (50Mhz clock domain)
reg   [26:0] reset_cnt = 0;
wire  resetn = &reset_cnt;
always @(posedge clk_50mhz) reset_cnt <= reset_cnt + !resetn;

// Generate the slow speed clock
localparam slow_clock_size = 23;
reg [slow_clock_size:0]  slow_clock;
wire clk_a1hz;
always @(posedge clk_50mhz) slow_clock <= slow_clock + 24'd1;
assign clk_a1hz = slow_clock[slow_clock_size];

// Generate the slow speed reset signal
reg   [4:0] slow_reset_cnt = 0;
wire  slow_resetn = &slow_reset_cnt;
always @(posedge clk_a1hz) slow_reset_cnt <= slow_reset_cnt + !slow_resetn;


reg [7:0] uart_din;
reg uart_we;
wire uart_tx_busy;
wire uart_rdy;
reg uart_rdy_clr;
wire [7:0] uart_dout;

uart  serial_uart(
	.din(uart_din),
	.wr_en(uart_we),
	.clk_50m(clk_50mhz),
	.tx(uart_tx),
	.tx_busy(uart_tx_busy),
	.rx(uart_rx),
	.rdy(uart_rdy),
	.rdy_clr(uart_rdy_clr),
	.dout(uart_dout)
);

localparam base_state = 0;
localparam do_read = 1;
localparam do_write = 2;
localparam finish_read_write = 3;

reg [2:0] state = base_state;

localparam debug_bytes = 4; // must be power of two
localparam debug_bytes_l2 = $clog2(debug_bytes);

reg [7:0]  data[0:debug_bytes - 1];
wire [7:0] data_wd[0:debug_bytes - 1];
reg [debug_bytes_l2:0] byte_output_count;
reg [7:0] input_byte;

assign data_wd[0] = 0;
//assign data_wd[1] = 1;
//assign data_wd[2] = 2;
//assign data_wd[3] = 3;
cpu my_cpu(slow_clock, slock_resetn, data_wd[1], data_wd[2], data_wd[3]);

always @(posedge clk_a1hz) begin
    data[0] <= data[0] + 1; // Delete this if you don't want a slow speed cycle counter, but it's highly recommended.
    data[1] <= data_wd[1]; 
    data[2] <= data_wd[2];
    data[3] <= data_wd[3];
end

// Program the 4 hex display lights to show the first two debug outputs.
hex_display hex_display_inst1(data[0][7:4], hex_display4);
hex_display hex_display_inst2(data[0][3:0], hex_display3);
hex_display hex_display_inst3(data[1][7:4], hex_display2);
hex_display hex_display_inst4(data[1][3:0], hex_display1);

always @(posedge clk_50mhz) begin
    if (!resetn) begin
        state <= base_state;
        byte_output_count <= 0;
    end
    else begin
		uart_rdy_clr <= 0;
		uart_we <= 0;

        // Basic state machine: read a byte if available, otherwise output next debug byte
        case (state)
        base_state: begin
				//if (uart_rdy)
            //   state <= do_read;
            //else
				if (!uart_tx_busy)
               state <= do_write;
            
        end
        do_read: begin
            uart_rdy_clr <= 1;
            input_byte <= uart_dout;
            state <= finish_read_write;
        end
        do_write: begin
            if (byte_output_count[debug_bytes_l2])
              uart_din <= 8'b11111111;
            else
              uart_din <= data[byte_output_count[debug_bytes_l2 - 1:0]];

		  	   byte_output_count <= byte_output_count + 1;
            uart_we <= 1;
            state <= finish_read_write;
        end
        finish_read_write: begin
				state <= base_state;
        end
        endcase 
    end
end


endmodule
